﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Editors;

namespace CustomComboEditorColumn
{
    public class AttributeViewModel : NotifyDataErrorInfoViewModelBase
    {
        #region Fields

        private AttributeModel m_Attribute;
        private ValueModel _EmptyValue = new ValueModel(string.Empty);

        #endregion Fields


        #region Properties

        public string Name
        {
            get
            {
                return m_Attribute.Name;
            }
        }

        public ObservableCollection<ValueModel> Values
        {
            get
            {
                return m_Attribute.SuggestedValues;
            }
        }

        public ValueModel SelectedValue
        {
            get
            {
                return _SelectedValue;
            }
            set
            {
                if (m_Attribute.Value != value)
                {
                    if (value != null)
                    {
                        //accept value value
                        m_Attribute.Value = value;
                        _SelectedValue = Values.FirstOrDefault(val => val.ValueDescription == m_Attribute.Value.ValueDescription);
                    }
                    else //delete value
                    {
                        m_Attribute.Value = _EmptyValue;
                        _SelectedValue = _EmptyValue;
                        RemoveErrorsForProperty("SelectedValue");
                    }

                    OnNotifyPropertyChanged("SelectedValue");
                }
            }
        }

        private ValueModel _SelectedValue;

        #endregion Properties


        #region Constructor

        public AttributeViewModel() { }

        public AttributeViewModel(AttributeModel attribute)
        {
            m_Attribute = attribute;
            _SelectedValue = Values.FirstOrDefault(val => val.ValueDescription == m_Attribute.Value.ValueDescription);
            if (_SelectedValue == null)
            {
                _SelectedValue = _EmptyValue;
            }
        }

        #endregion Constructor



        //min 2 alphanumerics 
        private static readonly Regex _AlphaNumericRegex = new Regex(@"^[a-zA-Z][a-zA-Z]+$");

        //min 1 number
        private static readonly Regex _NumberRegex = new Regex(@"^\d+$");

        private bool IsValid(ValueModel value)
        {
            bool isValid = true;
            string errorText = null;

            if ((m_Attribute.Name == "CompanyName") || (m_Attribute.Name == "Location") || (m_Attribute.Name == "Material"))
            {
                isValid = _AlphaNumericRegex.IsMatch(value.ValueDescription);
                errorText = "Not a string!";
            }

            if ((m_Attribute.Name == "Height") || (m_Attribute.Name == "Width"))
            {
                isValid = _NumberRegex.IsMatch(value.ValueDescription);
                errorText = "Not a Number";
            }

            if (!string.IsNullOrEmpty(errorText))
            {
                if (isValid)
                {
                    RemoveErrorsForProperty("SelectedValue");
                }
                else
                {
                    AddErrorForProperty("SelectedValue", errorText);
                }
            }

            return isValid;
        }

        public void AddItem(ComboItemAddingEventArgs<ComboEditorItem> e)
        {
            ValueModel value = e.Item.Data as ValueModel;

            if (!IsValid(value))
            {
                e.Cancel = true;
            }
        }
    }
}
