﻿using System;
using System.Collections;
using System.ComponentModel;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Editors;
using Infragistics.Controls.Grids;
using Infragistics.Controls.Grids.Primitives;

namespace CustomComboEditorColumn
{
    public class ComboBoxColumnContentProvider : ColumnContentProviderBase
    {
        TextBlock _tb;
        XamComboEditor _combobox;
        private CustomComboBoxColumn _Column;

        private bool m_NeedsRefresh;

        public ComboBoxColumnContentProvider()
        {
            _tb = new TextBlock();
            _combobox = new XamComboEditor();

            _combobox.ItemAdding += new EventHandler<ComboItemAddingEventArgs<ComboEditorItem>>(_combobox_ItemAdding);
        }

        void _combobox_ItemAdding(object sender, ComboItemAddingEventArgs<ComboEditorItem> e)
        {
            if (_Column != null)
            {
                _Column.OnXamComboEditorItemAdding(e);
            }
        }


        public override void ResetContent()
        {
            m_NeedsRefresh = true;
            base.ResetContent();
        }


        public override void AdjustDisplayElement(Cell cell)
        {
            if (m_NeedsRefresh)
            {
                CustomComboBoxColumn column = (CustomComboBoxColumn)cell.Column;

                Binding textBinding = new Binding();
                textBinding.Source = cell.Row.Data;

                if (column.DisplayMemberPath != null)
                    textBinding.Path =
                        new PropertyPath(string.Format("{0}.{1}", column.Key,
                                                       column.DisplayMemberPath));
                else
                    textBinding.Path = new PropertyPath(column.Key);

                textBinding.Mode = BindingMode.TwoWay;
                this._tb.SetBinding(TextBlock.TextProperty, textBinding);


                m_NeedsRefresh = false;

                base.AdjustDisplayElement(cell);
            }
        }

        public override FrameworkElement ResolveDisplayElement(Cell cell, Binding cellBinding)
        {
            CustomComboBoxColumn column = (CustomComboBoxColumn)cell.Column;

            Binding textBinding = new Binding();
            textBinding.Source = cell.Row.Data;

            if (column.DisplayMemberPath != null)
                textBinding.Path =
                    new PropertyPath(string.Format("{0}.{1}", column.Key,
                                     column.DisplayMemberPath));
            else
                textBinding.Path = new PropertyPath(column.Key);

            textBinding.Mode = BindingMode.TwoWay;
            this._tb.SetBinding(TextBlock.TextProperty, textBinding);


            return _tb;
        }

        public override object ResolveValueFromEditor(Cell cell)
        {
            return this._combobox.SelectedItem;
        }

        protected override FrameworkElement ResolveEditorControl(Cell cell, object editorValue, double availableWidth, double availableHeight, Binding editorBinding)
        {
            CustomComboBoxColumn column = (CustomComboBoxColumn)cell.Column;

            _Column = column;
            /*
             * column.ItemSource is always null. Therefore i found the ItemSourceMemberPath
             */
            if (column.ItemSource != null)
            {
                this._combobox.SetValue(XamComboEditor.ItemsSourceProperty, column.ItemSource);
            }
            else
            {
                Binding b = new Binding
                {
                    Source = cell.Row.Data,
                    Path = new PropertyPath(column.ItemSourceMemberPath),
                };
                this._combobox.SetBinding(XamComboEditor.ItemsSourceProperty, b);
            }

            Binding selectedItemBinding = new Binding();
            selectedItemBinding.Source = cell.Row.Data;
            selectedItemBinding.Path = new PropertyPath(column.Key);
            selectedItemBinding.Mode = BindingMode.TwoWay;
            this._combobox.SetBinding(XamComboEditor.SelectedItemProperty, selectedItemBinding);

            this._combobox.SetValue(XamComboEditor.DisplayMemberPathProperty, column.DisplayMemberPath);
            this._combobox.SetValue(XamComboEditor.EmptyTextProperty, column.EmptyText);

            this._combobox.SetValue(XamComboEditor.CustomValueEnteredActionProperty, column.CustomValueEnteredAction);

            this._combobox.SetValue(XamComboEditor.AutoCompleteProperty, column.AutoComplete);
            this._combobox.SetValue(XamComboEditor.AutoCompleteDelayProperty, column.AutoCompleteDelay);


            this._combobox.Style = cell.EditorStyleResolved;

            _combobox.DataContext = cell.Row.Data;

            return this._combobox;
        }


    }

        public class CustomComboBoxColumn : EditableColumn
        {
            protected override ColumnContentProviderBase GenerateContentProvider()
            {
                return new ComboBoxColumnContentProvider();
            }


            #region XamComboEditorItemAdding

            /// <summary>
            /// Called before the XamComboEditorItemAdding event occurs.
            /// </summary>
            protected internal void OnXamComboEditorItemAdding(ComboItemAddingEventArgs<ComboEditorItem> e)
            {
                if (XamComboEditorItemAdding == null) { return; }

                XamComboEditorItemAdding(this, e);
            }

            /// <summary>
            /// Occurs when an item is going to be added to the underlying ComboEditorItemCollection of the Infragistics.Controls.Editors.XamComboEditor.
            /// </summary>
            public event EventHandler<ComboItemAddingEventArgs<ComboEditorItem>> XamComboEditorItemAdding;

            #endregion XamComboEditorItemAdding


            public static readonly DependencyProperty ItemSourceProperty =
                DependencyProperty.Register("ItemSource",
                    typeof(IEnumerable),
                    typeof(CustomComboBoxColumn),
                    new PropertyMetadata(
                        new PropertyChangedCallback(ItemSourceChanged)));

            public IEnumerable ItemSource
            {
                get { return (IEnumerable)this.GetValue(ItemSourceProperty); }
                set { this.SetValue(ItemSourceProperty, value); }
            }

            private static void ItemSourceChanged(DependencyObject obj,
                DependencyPropertyChangedEventArgs e)
            {
                CustomComboBoxColumn col = (CustomComboBoxColumn)obj;
                col.OnPropertyChanged("ItemSource");
            }


            /*
             * with a binding to Itemssource i could not get it to bind.
             * Therefor ItemsSourceMemberPath is needed to make a binding in the ComboBoxColumnContentProvider
             */
            #region ItemSourceMemberPath

            /// <summary>
            /// Identifies the <see cref="ItemSourceMemberPath"/> dependency property. 
            /// </summary>
            public static readonly DependencyProperty ItemSourceMemberPathProperty = DependencyProperty.Register("ItemSourceMemberPath", typeof(String), typeof(CustomComboBoxColumn), null);


            /// <summary>
            /// Gets or sets the string containing the path to the ItemsSource.
            /// </summary>
            public String ItemSourceMemberPath
            {
                get
                {
                    return (String)GetValue(ItemSourceMemberPathProperty);
                }
                set
                {
                    SetValue(ItemSourceMemberPathProperty, value);
                }
            }

            #endregion ItemSourceMemberPath


            public static readonly DependencyProperty DisplayMemberPathProperty =
                DependencyProperty.Register("DisplayMemberPath",
                    typeof(string),
                    typeof(CustomComboBoxColumn),
                    new PropertyMetadata(
                        new PropertyChangedCallback(DisplayMemberPathChanged)));

            public string DisplayMemberPath
            {
                get { return (string)this.GetValue(DisplayMemberPathProperty); }
                set { this.SetValue(DisplayMemberPathProperty, value); }
            }

            private static void DisplayMemberPathChanged(DependencyObject obj,
                DependencyPropertyChangedEventArgs e)
            {
                CustomComboBoxColumn col = (CustomComboBoxColumn)obj;
                col.OnPropertyChanged("DisplayMemberPath");
            }

            public static readonly DependencyProperty ItemTemplateProperty =
                DependencyProperty.Register("ItemTemplate",
                    typeof(DataTemplate),
                    typeof(CustomComboBoxColumn),
                    new PropertyMetadata(
                        new PropertyChangedCallback(ItemTemplateChanged)));

            public string ItemTemplate
            {
                get { return (string)this.GetValue(ItemTemplateProperty); }
                set { this.SetValue(ItemTemplateProperty, value); }
            }

            private static void ItemTemplateChanged(DependencyObject obj,
                DependencyPropertyChangedEventArgs e)
            {
                CustomComboBoxColumn col = (CustomComboBoxColumn)obj;
                col.OnPropertyChanged("ItemTemplate");

            }


            #region EmptyText

            /// <summary>
            /// Identifies the <see cref="EmptyText"/> dependency property. 
            /// </summary>
            public static readonly DependencyProperty EmptyTextProperty = DependencyProperty.Register("EmptyText", typeof(string), typeof(CustomComboBoxColumn), null);

            /// <summary>
            /// Gets or sets the text that should be displayed when the editor doesn't have anything selected.  
            /// </summary>
            public string EmptyText
            {
                get
                {
                    return (string)GetValue(EmptyTextProperty);
                }
                set
                {
                    SetValue(EmptyTextProperty, value);
                }
            }

            #endregion



            #region AutoComplete

            /// <summary>
            /// Identifies the <see cref="AutoComplete"/> dependency property. 
            /// </summary>
            public static readonly DependencyProperty AutoCompleteProperty = DependencyProperty.Register("AutoComplete", typeof(bool), typeof(CustomComboBoxColumn), null);

            /// <summary>
            /// Gets or sets whether or not the combo should look through the data source while the user is typing, and hilight the rest of the text that matches with what they are currently typing.  
            /// </summary>
            public bool AutoComplete
            {
                get
                {
                    return (bool)GetValue(AutoCompleteProperty);
                }
                set
                {
                    SetValue(AutoCompleteProperty, value);
                }
            }

            #endregion

            #region AutoCompleteDelay

            /// <summary>
            /// Identifies the <see cref="AutoCompleteDelay"/> dependency property. 
            /// </summary>
            public static readonly DependencyProperty AutoCompleteDelayProperty = DependencyProperty.Register("AutoCompleteDelay", typeof(int), typeof(CustomComboBoxColumn), null);

            /// <summary>
            /// Gets or sets the minimum delay, in milliseconds, after text is typed in the text box before the Infragistics.Controls.Editors.XamComboEditor 
            /// control populates the list of possible matches in the drop-down.  
            /// </summary>
            public int AutoCompleteDelay
            {
                get
                {
                    return (int)GetValue(AutoCompleteDelayProperty);
                }
                set
                {
                    SetValue(AutoCompleteDelayProperty, value);
                }
            }

            #endregion

            #region CustomValueEnteredAction

            /// <summary>
            /// Identifies the <see cref="CustomValueEnteredAction"/> dependency property. 
            /// </summary>
            public static readonly DependencyProperty CustomValueEnteredActionProperty = DependencyProperty.Register("CustomValueEnteredAction", typeof(CustomValueEnteredActions), typeof(CustomComboBoxColumn), null);

            /// <summary>
            /// Gets or sets what action should occur when entering data into the combo editor that doesn't exist in the underlying ItemsSource.  
            /// </summary>
            public CustomValueEnteredActions CustomValueEnteredAction
            {
                get
                {
                    return (CustomValueEnteredActions)GetValue(CustomValueEnteredActionProperty);
                }
                set
                {
                    SetValue(CustomValueEnteredActionProperty, value);
                }
            }

            #endregion
        }
}
